@@ -4,20 +4,22 @@
 
 import os
 import sys
+import ast
 import argparse
 import logging
 
-from doorstop import CLI, VERSION
 from doorstop.gui.main import _run as gui
 from doorstop.core.tree import build
-from doorstop.core import report
+from doorstop.core import report, importer
 from doorstop import common
-from doorstop.common import DoorstopError, HelpFormatter, WarningFormatter
+from doorstop.common import HelpFormatter, WarningFormatter, DoorstopError
 from doorstop import settings
 
 
 def main(args=None):  # pylint: disable=R0915
     """Process command-line arguments and run the program."""
+    from doorstop import CLI, VERSION
+
     # Shared options
     debug = argparse.ArgumentParser(add_help=False)
     debug.add_argument('-j', '--project', metavar='PATH',
@@ -31,10 +33,14 @@ def main(args=None):  # pylint: disable=R0915
     parser = argparse.ArgumentParser(prog=CLI, description=__doc__, **shared)
     parser.add_argument('-F', '--no-reformat', action='store_true',
                         help="do not reformat item files during validation")
+    parser.add_argument('-r', '--reorder', action='store_true',
+                        help="reorder document levels during validation")
+    parser.add_argument('-L', '--no-level-check', action='store_true',
+                        help="do not validate document levels")
     parser.add_argument('-R', '--no-ref-check', action='store_true',
                         help="do not validate external file references")
-    parser.add_argument('-L', '--no-rlinks-check', action='store_true',
-                        help="do not validate reverse links")
+    parser.add_argument('-C', '--no-child-check', action='store_true',
+                        help="do not validate child (reverse) links")
     parser.add_argument('-g', '--gui', action='store_true',
                         help="launch the graphical user interface")
     subs = parser.add_subparsers(help="", dest='command', metavar="<command>")
@@ -44,7 +50,7 @@ def main(args=None):  # pylint: disable=R0915
                           help="create a new document directory",
                           **shared)
     sub.add_argument('prefix', help="document prefix for new item IDs")
-    sub.add_argument('root', help="path to a directory for item files")
+    sub.add_argument('path', help="path to a directory for item files")
     sub.add_argument('-p', '--parent', help="prefix for parent item IDS")
     sub.add_argument('-d', '--digits', help="number of digits in item IDs")
 
@@ -54,6 +60,7 @@ def main(args=None):  # pylint: disable=R0915
                           **shared)
     sub.add_argument('prefix',
                      help="document prefix for the new item")
+    sub.add_argument('-l', '--level', help="desired item level (e.g. 1.2.3)")
 
     # Remove subparser
     sub = subs.add_parser('remove',
@@ -88,6 +95,19 @@ def main(args=None):  # pylint: disable=R0915
     sub.add_argument('-t', '--tool', metavar='PROGRAM',
                      help="text editor to open the document item")
 
+    # Import subparser
+    sub = subs.add_parser('import',
+                          help="import an existing document or item",
+                          **shared)
+    sub.add_argument('-d', '--document', nargs=2, metavar='ARG',
+                     help="import an existing document by: PREFIX PATH")
+    sub.add_argument('-i', '--item', nargs=2, metavar='ARG',
+                     help="import an existing item by: PREFIX ID")
+    sub.add_argument('-p', '--parent', metavar='PREFIX',
+                     help="parent document prefix for imported document")
+    sub.add_argument('-a', '--attrs', metavar='DICT',
+                     help="dictionary of item attributes to import")
+
     # Publish subparser
     sub = subs.add_parser('publish',
                           help="publish a document as text or another format",
@@ -103,6 +123,8 @@ def main(args=None):  # pylint: disable=R0915
                      help="output HTML (default for 'all')")
     sub.add_argument('-w', '--width', type=int,
                      help="limit line width on text output")
+    sub.add_argument('-C', '--with-child-links', action='store_true',
+                     help="include child links in published documents")
 
     # Parse arguments
     args = parser.parse_args(args=args)
@@ -110,6 +132,9 @@ def main(args=None):  # pylint: disable=R0915
     # Configure logging
     _configure_logging(args.verbose)
 
+    # Configure settings
+    _configure_settings(args)
+
     # Run the program
     if args.gui:
         logging.debug("launching GUI...")
@@ -170,6 +195,24 @@ def _configure_logging(verbosity=0):
         common.VERBOSITY = verbosity
 
 
+def _configure_settings(args):
+    """Update settings based on the command-line options."""
+    # Parse common settings
+    if args.no_reformat is not None:
+        settings.REFORMAT = not args.no_reformat
+    if args.reorder is not None:
+        settings.REORDER = args.reorder
+    if args.no_level_check is not None:
+        settings.CHECK_LEVELS = not args.no_level_check
+    if args.no_ref_check is not None:
+        settings.CHECK_REF = not args.no_ref_check
+    if args.no_child_check is not None:
+        settings.CHECK_CHILD_LINKS = not args.no_child_check
+    # Parse subcommand settings
+    if 'with_child_links' in args and args.with_child_links is not None:
+        settings.PUBLISH_CHILD_LINKS = args.with_child_links
+
+
 def _run(args, cwd, err):  # pylint: disable=W0613
     """Process arguments and run the `doorstop` subcommand.
 
@@ -178,26 +221,17 @@ def _run(args, cwd, err):  # pylint: disable=W0613
     @param err: function to call for CLI errors
 
     """
-    # Configure validation settings
-    if args.no_reformat is not None:
-        settings.REFORMAT = not args.no_reformat
-    if args.no_ref_check is not None:
-        settings.CHECK_REF = not args.no_ref_check
-    if args.no_rlinks_check is not None:
-        settings.CHECK_RLINKS = not args.no_rlinks_check
-
-    # Validate the tree
     try:
         tree = build(cwd, root=args.project)
         tree.load()
-        valid = tree.valid()
+        valid = tree.validate()
     except DoorstopError as error:
         logging.error(error)
         return False
-    else:
-        if tree and valid:
-            print("valid tree: {}".format(tree))
-        return valid
+
+    if tree and valid:
+        print("valid tree: {}".format(tree))
+    return valid
 
 
 def _run_new(args, cwd, _):
@@ -210,14 +244,15 @@ def _run_new(args, cwd, _):
     """
     try:
         tree = build(cwd, root=args.project)
-        document = tree.new(args.root, args.prefix,
-                            parent=args.parent, digits=args.digits)
+        document = tree.new_document(args.path, args.prefix,
+                                     parent=args.parent, digits=args.digits)
     except DoorstopError as error:
         logging.error(error)
         return False
-    else:
-        print("created document: {}".format(document.prefix_relpath))
-        return True
+
+    print("created document: {} ({})".format(document.prefix,
+                                             document.relpath))
+    return True
 
 
 def _run_add(args, cwd, _):
@@ -230,13 +265,13 @@ def _run_add(args, cwd, _):
     """
     try:
         tree = build(cwd, root=args.project)
-        item = tree.add(args.prefix)
+        item = tree.add_item(args.prefix, level=args.level)
     except DoorstopError as error:
         logging.error(error)
         return False
-    else:
-        print("added item: {}".format(item.id_relpath))
-        return True
+
+    print("added item: {} ({})".format(item.id, item.relpath))
+    return True
 
 
 def _run_remove(args, cwd, _):
@@ -249,13 +284,13 @@ def _run_remove(args, cwd, _):
     """
     try:
         tree = build(cwd, root=args.project)
-        item = tree.remove(args.id)
+        item = tree.remove_item(args.id)
     except DoorstopError as error:
         logging.error(error)
         return False
-    else:
-        print("removed item: {}".format(item.id_relpath))
-        return True
+
+    print("removed item: {} ({})".format(item.id, item.relpath))
+    return True
 
 
 def _run_link(args, cwd, _):
@@ -268,14 +303,14 @@ def _run_link(args, cwd, _):
     """
     try:
         tree = build(cwd, root=args.project)
-        child, parent = tree.link(args.child, args.parent)
+        child, parent = tree.link_items(args.child, args.parent)
     except DoorstopError as error:
         logging.error(error)
         return False
-    else:
-        print("linked item: {} -> {}".format(child.id_relpath,
-                                             parent.id_relpath))
-        return True
+
+    msg = "linked items: {} ({}) -> {} ({})"
+    print(msg.format(child.id, child.relpath, parent.id, parent.relpath))
+    return True
 
 
 def _run_unlink(args, cwd, _):
@@ -288,14 +323,14 @@ def _run_unlink(args, cwd, _):
     """
     try:
         tree = build(cwd, root=args.project)
-        child, parent = tree.unlink(args.child, args.parent)
+        child, parent = tree.unlink_items(args.child, args.parent)
     except DoorstopError as error:
         logging.error(error)
         return False
-    else:
-        print("unlinked item: {} -> {}".format(child.id_relpath,
-                                               parent.id_relpath))
-        return True
+
+    msg = "unlinked items: {} ({}) -> {} ({})"
+    print(msg.format(child.id, child.relpath, parent.id, parent.relpath))
+    return True
 
 
 def _run_edit(args, cwd, _):
@@ -308,13 +343,48 @@ def _run_edit(args, cwd, _):
     """
     try:
         tree = build(cwd, root=args.project)
-        item = tree.edit(args.id, tool=args.tool, launch=True)
+        item = tree.edit_item(args.id, tool=args.tool, launch=True)
     except DoorstopError as error:
         logging.error(error)
         return False
+
+    print("opened item: {} ({})".format(item.id, item.relpath))
+    return True
+
+
+def _run_import(args, _, err):
+    """Process arguments and run the `doorstop import` subcommand.
+
+    @param args: Namespace of CLI arguments
+    @param cwd: current working directory
+    @param err: function to call for CLI errors
+
+    """
+    document = item = None
+
+    try:
+        if args.document:
+            prefix, path = args.document
+            document = importer.new_document(prefix, path, parent=args.parent)
+        elif args.item:
+            prefix, identifier = args.item
+            attrs = ast.literal_eval(args.attrs) if args.attrs else None
+            item = importer.add_item(prefix, identifier, attrs=attrs)
+        else:
+            err("specify '--document' or '--item' to import")
+    except DoorstopError as error:
+        logging.error(error)
+        return False
+
+    if document:
+        name = document.prefix
+        relpath = document.relpath
     else:
-        print("opened item: {}".format(item.id_relpath))
-        return True
+        assert item
+        name = item.id
+        relpath = item.relpath
+    print("imported: {} ({})".format(name, relpath))
+    return True
 
 
 def _run_publish(args, cwd, err):
@@ -325,7 +395,12 @@ def _run_publish(args, cwd, err):
     @param err: function to call for CLI errors
 
     """
+    # Parse arguments
     publish_tree = args.prefix == 'all'
+    ext = _get_extension(args, tree=publish_tree)
+    html = ext == '.html'
+
+    # Publish documents
     try:
         tree = build(cwd, root=args.project)
         if publish_tree:
@@ -335,49 +410,56 @@ def _run_publish(args, cwd, err):
     except DoorstopError as error:
         logging.error(error)
         return False
-    else:
 
-        # Set base arguments
-        kwargs = {'ignored': tree.vcs.ignored}
-        if args.width:
-            kwargs['width'] = args.width
+    # Set publishing arguments
+    kwargs = {'ignored': tree.vcs.ignored}
+    if args.width:
+        kwargs['width'] = args.width
 
-        # Set file extension
-        if args.path:
-            if publish_tree:
-                ext = '.html'
-            else:
-                ext = os.path.splitext(args.path)[-1]
-        if args.text:
-            ext = '.txt'
-        elif args.markdown:
-            ext = '.md'
-        elif args.html:
-            ext = '.html'
-        elif not args.path:
-            ext = '.txt'
-
-        # Publish documents
-        if args.path:
-            if publish_tree:
-                print("publishing tree to {}...".format(args.path))
-                for document in documents:
-                    path = os.path.join(args.path, document.prefix + ext)
-                    print("publishing {} to {}...".format(document, path))
-                    report.publish(document, path, ext, **kwargs)
-                if ext == '.html':
-                    report.index(args.path)
-            else:
-                print("publishing {} to {}...".format(documents[0], args.path))
-                report.publish(documents[0], args.path, ext, **kwargs)
+    # Write to output file(s)
+    if args.path:
+        if publish_tree:
+            print("publishing tree to {}...".format(args.path))
+            for document in documents:
+                path = os.path.join(args.path, document.prefix + ext)
+                print("publishing {} to {}...".format(document, path))
+                report.publish(document, path, ext, **kwargs)
+            if html:
+                report.index(args.path)
         else:
-            if publish_tree:
-                err("only single documents can be displayed")
-            for line in report.lines(documents[0], ext, **kwargs):
-                print(line)
+            print("publishing {} to {}...".format(documents[0], args.path))
+            report.publish(documents[0], args.path, ext, **kwargs)
 
-        return True
+    # Display to standard output
+    else:
+        if publish_tree:
+            err("only single documents can be displayed")
+        for line in report.lines(documents[0], ext, **kwargs):
+            print(line)
+
+    return True
 
 
-if __name__ == '__main__':  # pragma: no cover, manual test
+def _get_extension(args, tree=False):
+    """Determine the output file extensions from input arguments."""
+    # Get the argument from a provided output path
+    if args.path:
+        if tree:
+            ext = '.html'
+        else:
+            ext = os.path.splitext(args.path)[-1]
+    # Override the extension if a format is specified
+    if args.text:
+        ext = '.txt'
+    elif args.markdown:
+        ext = '.md'
+    elif args.html:
+        ext = '.html'
+    elif not args.path:
+        ext = '.txt'
+
+    return ext
+
+
+if __name__ == '__main__':  # pragma: no cover (manual test)
     main()
