@@ -4,28 +4,29 @@ import os
 import re
 import logging
 
-from doorstop.core import base
+from doorstop.core.base import BaseValidatable
 from doorstop.core.base import auto_load, auto_save, BaseFileObject
+from doorstop.core.types import ID, Text, Level
 from doorstop import common
 from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
 from doorstop import settings
 
 
-class Item(BaseFileObject):  # pylint: disable=R0904
+class Item(BaseValidatable, BaseFileObject):  # pylint: disable=R0902,R0904
 
     """Represents an item file with linkable text."""
 
     EXTENSIONS = '.yml', '.yaml'
 
-    DEFAULT_LEVEL = (1, 0)
+    DEFAULT_LEVEL = Level('1.0')
     DEFAULT_ACTIVE = True
     DEFAULT_NORMATIVE = True
     DEFAULT_DERIVED = False
-    DEFAULT_TEXT = ""
+    DEFAULT_TEXT = Text()
     DEFAULT_REF = ""
 
-    def __init__(self, path, root=os.getcwd()):
-        """Load an item from an existing file.
+    def __init__(self, path, root=os.getcwd(), **kwargs):
+        """Initialize an item from an existing file.
 
         @param path: path to Item file
         @param root: path to root of project
@@ -39,7 +40,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         filename = os.path.basename(path)
         name, ext = os.path.splitext(filename)
         try:
-            split_id(name)
+            ID(name).prefix, ID(name).number
         except DoorstopError:
             msg = "invalid item filename: {}".format(filename)
             raise DoorstopError(msg) from None
@@ -50,6 +51,9 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         # Initialize the item
         self.path = path
         self.root = root
+        self.document = kwargs.get('document')
+        self.tree = kwargs.get('tree')
+        self.auto = kwargs.get('auto', Item.auto)
         # Set default values
         self._data['level'] = Item.DEFAULT_LEVEL
         self._data['active'] = Item.DEFAULT_ACTIVE
@@ -60,13 +64,13 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         self._data['links'] = set()
 
     def __repr__(self):
-        return "Item({})".format(repr(self.path))
+        return "Item('{}')".format(self.path)
 
     def __str__(self):
         if common.VERBOSITY < common.STR_VERBOSITY:
-            return self.id
+            return str(self.id)
         else:
-            return self.id_relpath
+            return "{} ({})".format(self.id, self.relpath)
 
     def __eq__(self, other):
         return isinstance(other, self.__class__) and self.path == other.path
@@ -78,30 +82,31 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         return self.level < other.level
 
     @staticmethod
-    def new(path, root, prefix, sep, digits, number, level=None, auto=None):  # pylint: disable=R0913
-        """Create a new item.
+    def new(tree, document, path, root, identifier, level=None, auto=None):  # pylint: disable=R0913
+        """Internal method to create a new item.
+
+        @param tree: reference to the tree that contains this item
+        @param document: reference to document that contains this item
 
         @param path: path to directory for the new item
         @param root: path to root of the project
-        @param prefix: prefix for the new item
-        @param sep: separator between prefix and number
-        @param digits: number of digits for the new document
-        @param number: number for the new item
+        @param identifier: ID for the new item
+
         @param level: level for the new item
-        @param auto: enables automatic save
+        @param auto: automatically save the item
 
         @raise DoorstopError: if the item already exists
 
+        @return: new Item
+
         """
-        identifier = join_id(prefix, sep, number, digits)
-        filename = identifier + Item.EXTENSIONS[0]
+        filename = str(identifier) + Item.EXTENSIONS[0]
         path2 = os.path.join(path, filename)
         # Create the initial item file
         logging.debug("creating item file at {}...".format(path2))
         Item._new(path2, name='item')
         # Initialize the item
-        item = Item(path2, root=root)
-        item.auto = False
+        item = Item(path2, root=root, document=document, tree=tree, auto=False)
         item.level = level if level is not None else item.level
         if auto or (auto is None and Item.auto):
             item.save()
@@ -120,7 +125,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         # Store parsed data
         for key, value in data.items():
             if key == 'level':
-                self._data['level'] = load_level(value)
+                self._data['level'] = Level(value)
             elif key == 'active':
                 self._data['active'] = bool(value)
             elif key == 'normative':
@@ -128,14 +133,14 @@ class Item(BaseFileObject):  # pylint: disable=R0904
             elif key == 'derived':
                 self._data['derived'] = bool(value)
             elif key == 'text':
-                self._data['text'] = load_text(value)
+                self._data['text'] = Text(value)
             elif key == 'ref':
                 self._data['ref'] = value.strip()
             elif key == 'links':
-                self._data['links'] = set(value)
+                self._data['links'] = set(ID(v) for v in value)
             else:
                 if isinstance(value, str):
-                    value = load_text(value)
+                    value = Text(value)
                 self._data[key] = value
         # Set meta attributes
         self._loaded = True
@@ -147,20 +152,20 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         data = {}
         for key, value in self._data.items():
             if key == 'level':
-                data['level'] = save_level(value)
+                data['level'] = value.yaml
             elif key == 'text':
-                data['text'] = save_text(self._data['text'])
+                data['text'] = value.yaml
             elif key == 'ref':
                 data['ref'] = value.strip()
             elif key == 'links':
-                data['links'] = sorted(value)
+                data['links'] = sorted(str(v) for v in value)
             else:
                 if isinstance(value, str):
                     # length of "key_text: value_text"
                     lenth = len(key) + 2 + len(value)
                     if lenth > settings.MAX_LINE_LENTH or '\n' in value:
                         end = '\n' if value.endswith('\n') else ''
-                        value = save_text(value, end=end)
+                        value = Text.save_text(value, end=end)
                 data[key] = value
         # Dump the data to YAML
         text = self._dump(data)
@@ -175,29 +180,18 @@ class Item(BaseFileObject):  # pylint: disable=R0904
     @property
     def id(self):  # pylint: disable=C0103
         """Get the item's ID."""
-        return os.path.splitext(os.path.basename(self.path))[0]
-
-    @property
-    def relpath(self):
-        """Get the item's relative path string."""
-        relpath = os.path.relpath(self.path, self.root)
-        return "@{}{}".format(os.sep, relpath)
-
-    # TODO: think of a better name for this property
-    @property
-    def id_relpath(self):
-        """Get the item's ID + relative path string."""
-        return "{} ({})".format(self.id, self.relpath)
+        filename = os.path.basename(self.path)
+        return ID(os.path.splitext(filename)[0])
 
     @property
     def prefix(self):
         """Get the item ID's prefix."""
-        return split_id(self.id)[0]
+        return self.id.prefix
 
     @property
     def number(self):
         """Get the item ID's number."""
-        return split_id(self.id)[1]
+        return self.id.number
 
     @property
     @auto_load
@@ -210,15 +204,12 @@ class Item(BaseFileObject):  # pylint: disable=R0904
     @auto_load
     def level(self, value):
         """Set the item's level."""
-        self._data['level'] = load_level(value)
+        self._data['level'] = Level(value)
 
     @property
     def depth(self):
         """Get the item's heading order based on it's level."""
-        level = list(self.level)
-        while level[-1] == 0:
-            del level[-1]
-        return len(level)
+        return len(self.level)
 
     @property
     @auto_load
@@ -289,7 +280,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         Headings have a level that ends in zero and are non-normative.
 
         """
-        return self.level[-1] == 0 and not self.normative
+        return self.level.heading and not self.normative
 
     @heading.setter
     @auto_save
@@ -298,10 +289,10 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         """Set the item's heading status."""
         heading = bool(value)
         if heading and not self.heading:
-            self.level = list(self.level) + [0]
+            self.level.heading = True
             self.normative = False
         elif not heading and self.heading:
-            self.level = list(self.level)[:-1]
+            self.level.heading = False
             self.normative = True
 
     @property
@@ -315,7 +306,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
     @auto_load
     def text(self, value):
         """Set the item's text."""
-        self._data['text'] = str(value) if value else ""
+        self._data['text'] = Text(value)
 
     @property
     @auto_load
@@ -348,56 +339,51 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         """Set the list of item IDs this item links to."""
         self._data['links'] = set(value)
 
+    @property
+    def parent_links(self):
+        """Get a list of the item IDs this item links to."""
+        return self.links  # alias
+
+    @parent_links.setter
+    def parent_links(self, value):
+        """Set the list of item IDs this item links to."""
+        self.links = value  # alias
+
     # actions ################################################################
 
     @auto_save
     @auto_load
-    def add_link(self, item):
-        """Add a new link to another item ID."""
-        self._data['links'].add(item)
+    def link(self, value):
+        """Add a new link to another item ID.
+
+        @param value: item or ID
+
+        """
+        identifier = ID(value)
+        self._data['links'].add(identifier)
 
     @auto_save
     @auto_load
-    def remove_link(self, item):
-        """Remove an existing link by item ID."""
-        try:
-            self._data['links'].remove(item)
-        except KeyError:
-            logging.warning("link to {0} does not exist".format(item))
+    def unlink(self, value):
+        """Remove an existing link by item ID.
 
-    def valid(self, document=None, tree=None):
-        """Check the item for validity.
-
-        @param document: Document containing the item
-        @param tree: Tree containing the item
-
-        @return: indication that the item is valid
+        @param value: item or ID
 
         """
-        # TODO: refactor: this could be common code with Item/Document/Tree
-        valid = True
-        # Display all issues
-        for issue in self.issues(document=document, tree=tree):
-            if isinstance(issue, DoorstopInfo):
-                logging.info(issue)
-            elif isinstance(issue, DoorstopWarning):
-                logging.warning(issue)
-            else:
-                assert isinstance(issue, DoorstopError)
-                logging.error(issue)
-                valid = False
-        # Return the result
-        return valid
+        identifier = ID(value)
+        try:
+            self._data['links'].remove(identifier)
+        except KeyError:
+            logging.warning("link to {0} does not exist".format(identifier))
 
-    def issues(self, document=None, tree=None):
+    def get_issues(self, **kwargs):
         """Yield all the item's issues.
 
-        @param document: Document containing the item
-        @param tree: Tree containing the item
-
         @return: generator of DoorstopError, DoorstopWarning, DoorstopInfo
 
         """
+        assert kwargs.get('document_hook') is None
+        assert kwargs.get('item_hook') is None
         logging.info("checking item {}...".format(self))
         # Verify the file can be parsed
         self.load()
@@ -414,27 +400,30 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         # Check external references
         if settings.CHECK_REF:
             try:
-                self.find_ref(ignored=tree.vcs.ignored if tree else None)
+                # TODO: find_ref should get 'self.tree.vcs.ignored' internally
+                self.find_ref(ignored=self.tree.vcs.ignored
+                              if self.tree else None)
             except DoorstopError as exc:
                 yield exc
         # Check links
         if not self.normative and self.links:
             yield DoorstopWarning("non-normative, but has links")
         # Check links against the document
-        if document:
-            yield from self._issues_document(document)
+        if self.document:
+            yield from self._get_issues_document(self.document)
         # Check links against the tree
-        if tree:
-            yield from self._issues_tree(tree)
+        if self.tree:
+            yield from self._get_issues_tree(self.tree)
         # Check links against both document and tree
-        if document and tree:
-            yield from self._issues_both(document, tree)
+        if self.document and self.tree:
+            yield from self._get_issues_both(self.document, self.tree)
         # Reformat the file
         if settings.REFORMAT:
             self.save()
 
-    def _issues_document(self, document):
+    def _get_issues_document(self, document):
         """Yield all the item's issues against its document."""
+        logging.debug("getting issues against document: {}".format(document))
         # Verify an item's ID matches its document's prefix
         if self.prefix != document.prefix:
             msg = "prefix differs from document ({})".format(document.prefix)
@@ -448,20 +437,22 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         # Verify an item's links are to the correct parent
         for identifier in self.links:
             try:
-                prefix = split_id(identifier)[0]
+                prefix = identifier.prefix
             except DoorstopError:
                 msg = "invalid ID in links: {}".format(identifier)
                 yield DoorstopError(msg)
             else:
-                if prefix != document.parent:
+                if document.parent and prefix != document.parent:
                     # this is only 'info' because a document is allowed
                     # to contain items with a different prefix, but
                     # Doorstop will not create items like this
-                    msg = "linked to non-parent item: {}".format(identifier)
+                    msg = "parent is '{}', but linked to: {}".format(
+                        document.parent, identifier)
                     yield DoorstopInfo(msg)
 
-    def _issues_tree(self, tree):
+    def _get_issues_tree(self, tree):
         """Yield all the item's issues against its tree."""
+        logging.debug("getting issues against tree: {}".format(tree))
         # Verify an item's links are valid
         identifiers = set()
         for identifier in self.links:
@@ -485,14 +476,16 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         if settings.REFORMAT:
             self._data['links'] = identifiers
 
-    def _issues_both(self, document, tree):
+    def _get_issues_both(self, document, tree):
         """Yield all the item's issues against its document and tree."""
-        # Verify an item is being linked to (reverse links)
-        if settings.CHECK_RLINKS and self.normative:
-            rlinks, children = self.find_rlinks(document, tree, find_all=False)
-            if not rlinks:
-                for child in children:
-                    msg = "no links from child document: {}".format(child)
+        logging.debug("getting issues against both: {} & {}".format(document,
+                                                                    tree))
+        # Verify an item is being linked to (child links)
+        if settings.CHECK_CHILD_LINKS and self.normative:
+            items, documents = self._find_child_objects(find_all=False)
+            if not items:
+                for document in documents:
+                    msg = "no links from child document: {}".format(document)
                     yield DoorstopWarning(msg)
 
     def find_ref(self, root=None, ignored=None):
@@ -501,11 +494,11 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         @param root: override the path to the working copy (for testing)
         @param ignored: function to determine if a path should be skipped
 
+        @raise DoorstopError: when no reference is found
+
         @return: relative path to file, line number (when found in file)
                  relative path to file, None (when found as filename)
-                 None, None (when no ref)
-
-        @raise DoorstopError: when no ref is found
+                 None, None (when no reference set)
 
         """
         if not self.ref:
@@ -518,7 +511,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         regex = re.compile(pattern)
         logging.debug("search path: {}".format(root or self.root))
         for root, _, filenames in os.walk(root or self.root):
-            for filename in filenames:  # pragma: no cover, integration test
+            for filename in filenames:  # pragma: no cover (integration test)
                 path = os.path.join(root, filename)
                 relpath = os.path.relpath(path, self.root)
                 # Skip the item's file while searching
@@ -544,156 +537,81 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         msg = "external reference not found: {}".format(self.ref)
         raise DoorstopError(msg)
 
-    # TODO: should this only return IDs and add a find_children method?
-    def find_rlinks(self, document, tree, find_all=True):
+    def find_child_links(self, find_all=True):
         """Get a list of item IDs that link to this item (reverse links).
 
-        @param document: Document containing the item
-        @param tree: Tree containing the item
         @param find_all: find all items (not just the first) before returning
 
-        @return: list of found item IDs, list of all child Documents
+        @return: list of found item IDs
 
         """
-        rlinks = []
-        children = []
-        for document2 in tree:
-            if document2.parent == document.prefix:
-                children.append(document2)
-                # Search for reverse links unless we only need to find one
-                if not rlinks or find_all:
+        items, _ = self._find_child_objects(find_all=find_all)
+        identifiers = [item.id for item in items]
+        return identifiers
+
+    child_links = property(find_child_links)
+
+    def find_child_items(self, find_all=True):
+        """Get a list of items that link to this item.
+
+        @param find_all: find all items (not just the first) before returning
+
+        @return: list of found items
+
+        """
+        items, _ = self._find_child_objects(find_all=find_all)
+        return items
+
+    child_items = property(find_child_items)
+
+    def find_child_documents(self):
+        """Get a list of documents that should link to this item's document.
+
+        @return: list of found documents
+
+        """
+        _, documents = self._find_child_objects(find_all=False)
+        return documents
+
+    child_documents = property(find_child_documents)
+
+    def _find_child_objects(self, find_all=True):
+        """Get lists of child items and child documents.
+
+        @param find_all: find all items (not just the first) before returning
+
+        @return: list of found items, list of all child Documents
+
+        """
+        child_items = []
+        child_documents = []
+        # Check for parent references
+        if not self.document or not self.tree:
+            logging.warning("document and tree required to find children")
+            return child_items, child_documents
+        # Find child objects
+        for document2 in self.tree:
+            if document2.parent == self.document.prefix:
+                child_documents.append(document2)
+                # Search for child items unless we only need to find one
+                if not child_items or find_all:
                     for item in document2:
                         if self.id in item.links:
-                            rlinks.append(item.id)
+                            child_items.append(item)
                             if not find_all:
                                 break
-        if rlinks:
+        # Display found links
+        if child_items:
             if find_all:
-                msg = "reverse links: {}".format(', '.join(rlinks))
+                joined = ', '.join(str(i) for i in child_items)
+                msg = "child items: {}".format(joined)
             else:
-                msg = "first reverse link: {}".format(rlinks[0])
+                msg = "first child item: {}".format(child_items[0])
             logging.debug(msg)
-        return rlinks, children
+            joined = ', '.join(str(d) for d in child_documents)
+            logging.debug("child documents: {}".format(joined))
+        return child_items, child_documents
 
     def delete(self, path=None):
         """Delete the item."""
         super().delete(self.path)
-
-
-# attribute formatters #######################################################
-
-def split_id(text):
-    """Split an item's ID into a prefix and number.
-
-    >>> split_id('ABC00123')
-    ('ABC', 123)
-
-    >>> split_id('ABC.HLR_01-00123')
-    ('ABC.HLR_01', 123)
-
-    >>> split_id('REQ2-001')
-    ('REQ2', 1)
-
-    """
-    match = re.match(r"([\w.-]*\D)(\d+)", text)
-    if not match:
-        raise DoorstopError("invalid ID: {}".format(text))
-    prefix = match.group(1).rstrip(settings.SEP_CHARS)
-    number = int(match.group(2))
-    return prefix, number
-
-
-def join_id(prefix, sep, number, digits):
-    """Join the parts of an item's ID into an ID.
-
-    >>> join_id('ABC', '', 123, 5)
-    'ABC00123'
-
-    >>> join_id('REQ.H', '-', 42, 4)
-    'REQ.H-0042'
-
-    """
-    return "{}{}{}".format(prefix, sep, str(number).zfill(digits))
-
-
-def load_text(value):
-    r"""Convert dumped text to the original string.
-
-    >>> load_text("abc\ndef")
-    'abc def'
-
-    >>> load_text("list:\n\n- a\n- b\n")
-    'list:\n\n- a\n- b'
-
-    """
-    return base.join(value)
-
-
-def save_text(text, end='\n'):
-    """Break a string at sentences and dump as literal YAML with wrapping."""
-    return base.Literal(base.wrap(base.sbd(text, end=end)))
-
-
-def load_level(value):
-    """Convert an iterable, number, or level string to a tuple.
-
-    >>> load_level("1.2.3")
-    (1, 2, 3)
-
-    >>> load_level(['4', '5'])
-    (4, 5)
-
-    >>> load_level(4.2)
-    (4, 2)
-
-    >>> load_level([7, 0, 0])
-    (7, 0)
-
-    >>> load_level(1)
-    (1,)
-
-    """
-    # Correct for integers (e.g. 42) and floats (e.g. 4.2) in YAML
-    if isinstance(value, (int, float)):
-        value = str(value)
-
-    # Split strings by periods
-    if isinstance(value, str):
-        nums = value.split('.')
-    else:  # assume an iterable
-        nums = value
-
-    # Clean up multiple trailing zeros
-    parts = [int(n) for n in nums]
-    if parts[-1] == 0:
-        while parts[-1] == 0:
-            del parts[-1]
-        parts.append(0)
-
-    # Convert the level to a tuple
-    return tuple(parts)
-
-
-def save_level(parts):
-    """Convert a level's part into non-quoted YAML value.
-
-    >>> save_level((1,))
-    1
-
-    >>> save_level((1,0))
-    1.0
-
-    >>> save_level((1,0,0))
-    '1.0.0'
-
-    """
-    # Join the level's parts
-    level = '.'.join(str(n) for n in parts)
-
-    # Convert formats to cleaner YAML formats
-    if len(parts) == 1:
-        level = int(level)
-    elif len(parts) == 2 and not (level.endswith('0') and parts[-1]):
-        level = float(level)
-
-    return level
