@@ -35,7 +35,7 @@ def publish(obj, path, ext=None, ignored=None, **kwargs):
 
         # Publish report
         logging.info("publishing {}...".format(path))
-        with open(path, 'w') as outfile:  # pragma: no cover, integration test
+        with open(path, 'w') as outfile:  # pragma: no cover (integration test)
             for line in lines(obj, ext, ignored=ignored, **kwargs):
                 outfile.write(line + '\n')
     else:
@@ -113,11 +113,9 @@ def lines_text(obj, ignored=None, indent=8, width=79):
     @return: iterator of lines of text
 
     """
-    for item in _items(obj):
+    for item in _get_items(obj):
 
-        level = '.'.join(str(l) for l in item.level)
-        if level.endswith('.0') and len(level) > 3:
-            level = level[:-2]
+        level = _format_level(item.level)
 
         if item.heading:
 
@@ -135,7 +133,7 @@ def lines_text(obj, ignored=None, indent=8, width=79):
                 for line in item.text.splitlines():
                     yield from _chunks(line, width, indent)
 
-                    if not line:  # pragma: no cover - integration test
+                    if not line:  # pragma: no cover (integration test)
                         yield ""  # break between paragraphs
 
             # Reference
@@ -147,8 +145,18 @@ def lines_text(obj, ignored=None, indent=8, width=79):
             # Links
             if item.links:
                 yield ""  # break before links
-                links = "Links: " + ', '.join(item.links)
-                yield from _chunks(links, width, indent)
+                if settings.PUBLISH_CHILD_LINKS:
+                    label = "Parent links: "
+                else:
+                    label = "Links: "
+                slinks = label + ', '.join(str(l) for l in item.links)
+                yield from _chunks(slinks, width, indent)
+            if settings.PUBLISH_CHILD_LINKS:
+                links = item.find_child_links()
+                if links:
+                    yield ""  # break before links
+                    slinks = "Child links: " + ', '.join(str(l) for l in links)
+                    yield from _chunks(slinks, width, indent)
 
         yield ""  # break between items
 
@@ -169,12 +177,10 @@ def lines_markdown(obj, ignored=None):
     @return: iterator of lines of text
 
     """
-    for item in _items(obj):
+    for item in _get_items(obj):
 
         heading = '#' * item.depth
-        level = '.'.join(str(l) for l in item.level)
-        if level.endswith('.0') and len(level) > 3:
-            level = level[:-2]
+        level = _format_level(item.level)
 
         if item.heading:
 
@@ -199,13 +205,23 @@ def lines_markdown(obj, ignored=None):
             # Links
             if item.links:
                 yield ""  # break before links
-                links = '*' + "Links: " + ', '.join(item.links) + '*'
-                yield links
+                if settings.PUBLISH_CHILD_LINKS:
+                    label = "Parent links: "
+                else:
+                    label = "Links: "
+                slinks = label + ', '.join(str(l) for l in item.links)
+                yield '*' + slinks + '*'
+            if settings.PUBLISH_CHILD_LINKS:
+                links = item.find_child_links()
+                if links:
+                    yield ""  # break before links
+                    slinks = "Child links: " + ', '.join(str(l) for l in links)
+                    yield '*' + slinks + '*'
 
         yield ""  # break between items
 
 
-def _items(obj):
+def _get_items(obj):
     """Get an iterator of items from from an item, list, or document."""
     if hasattr(obj, 'items'):
         # a document
@@ -218,6 +234,14 @@ def _items(obj):
         return [obj]  # an item
 
 
+def _format_level(level):
+    """Convert a level to a string and keep zeros if not a top level."""
+    text = str(level)
+    if text.endswith('.0') and len(text) > 3:
+        text = text[:-2]
+    return text
+
+
 def _ref(item, ignored=None):
     """Format an external reference for publishing."""
     if settings.CHECK_REF:
@@ -251,7 +275,7 @@ def lines_html(obj, ignored=None):
         yield '<style type="text/css">'
         yield ''
         with open(CSS) as infile:
-            for line in infile:
+            for line in infile:  # pragma: no cover (integration test)
                 yield line
         yield '</style>'
         yield '</head>'
