@@ -5,11 +5,9 @@ from unittest.mock import patch, Mock, MagicMock
 
 import os
 
-from doorstop import common
 from doorstop.common import DoorstopError
 from doorstop.core.item import Item
 
-from doorstop.core.test import ENV, REASON
 from doorstop.core.test import FILES, EMPTY, EXTERNAL, MockFileObject
 
 
@@ -61,14 +59,14 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.item.save()
         self.item._write.assert_called_once_with(YAML_DEFAULT, self.item.path)
 
+    @patch('doorstop.common.VERBOSITY', 2)
     def test_str(self):
         """Verify an item can be converted to strings."""
-        common.VERBOSITY = 2
         self.assertEqual("RQ001", str(self.item))
 
+    @patch('doorstop.common.VERBOSITY', 3)
     def test_str_verbose(self):
         """Verify an item can be converted to strings in verbose mode."""
-        common.VERBOSITY = 3
         text = "RQ001 (@{}{})".format(os.sep, self.item.path)
         self.assertEqual(text, str(self.item))
 
@@ -99,11 +97,6 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.assertEqual(text, self.item.relpath)
         self.assertRaises(AttributeError, setattr, self.item, 'relpath', '.')
 
-    def test_id_relpath(self):
-        """Verify an item's ID and relative path string can be read."""
-        text = "{} (@{}{})".format(self.item.id, os.sep, self.item.path)
-        self.assertEqual(text, self.item.id_relpath)
-
     def test_prefix(self):
         """Verify an item's prefix can be read but not set."""
         self.assertEqual('RQ', self.item.prefix)
@@ -141,7 +134,13 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         """Verify an item's level can be set from text and read."""
         self.item.level = "4.2.0 "
         self.assertIn("level: 4.2.0\n", self.item._write.call_args[0][0])
-        self.assertEqual((4, 2, 0), self.item.level)
+        self.assertEqual((4, 2), self.item.level)
+
+    def test_level_from_text_2_digits(self):
+        """Verify an item's level can be set from text (2 digits) and read."""
+        self.item.level = "10.10"
+        self.assertIn("level: '10.10'\n", self.item._write.call_args[0][0])
+        self.assertEqual((10, 10), self.item.level)
 
     def test_level_from_float(self):
         """Verify an item's level can be set from a float and read."""
@@ -177,11 +176,9 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         """Verify an item's heading status can be set and read."""
         self.item.level = '1.1.1'
         self.item.heading = 1  # converted to True
-        self.assertEqual(0, self.item.level[-1])
         self.assertFalse(self.item.normative)
         self.assertTrue(self.item.heading)
         self.item.heading = 0  # converted to False
-        self.assertNotEqual(0, self.item.level[-1])
         self.assertTrue(self.item.normative)
         self.assertFalse(self.item.heading)
 
@@ -287,25 +284,54 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.item.set('text', "extended access")
         self.assertEqual("extended access", self.item.text)
 
-    def test_link_add(self):
+    def test_object_references_standalone(self):
+        """Verify a standalone item does not have object references."""
+        self.assertIs(None, self.item.document)
+        self.assertIs(None, self.item.tree)
+
+    def test_link(self):
         """Verify links can be added to an item."""
-        self.item.add_link('abc')
-        self.item.add_link('123')
+        self.item.link('abc')
+        self.item.link('123')
         self.assertEqual(['123', 'abc'], self.item.links)
 
-    def test_link_add_duplicate(self):
+    def test_link_duplicate(self):
         """Verify duplicate links are ignored."""
-        self.item.add_link('abc')
-        self.item.add_link('abc')
+        self.item.link('abc')
+        self.item.link('abc')
         self.assertEqual(['abc'], self.item.links)
 
-    def test_link_remove_duplicate(self):
+    def test_unlink_duplicate(self):
         """Verify removing a link twice is not an error."""
         self.item.links = ['123', 'abc']
-        self.item.remove_link('abc')
-        self.item.remove_link('abc')
+        self.item.unlink('abc')
+        self.item.unlink('abc')
         self.assertEqual(['123'], self.item.links)
 
+    def test_link_by_item(self):
+        """Verify links can be added to an item (by item)."""
+        path = os.path.join('path', 'to', 'ABC123.yml')
+        item = MockItem(path)
+        self.item.link(item)
+        self.assertEqual(['ABC123'], self.item.links)
+
+    def test_unlink_by_item(self):
+        """Verify links can be removed (by item)."""
+        path = os.path.join('path', 'to', 'ABC123.yml')
+        item = MockItem(path)
+        self.item.links = ['ABC123']
+        self.item.unlink(item)
+        self.assertEqual([], self.item.links)
+
+    def test_links_alias(self):
+        """Verify 'parent_links' is an alias for links."""
+        links1 = ['alias1']
+        links2 = ['alias2']
+        self.item.parent_links = links1
+        self.assertEqual(links1, self.item.links)
+        self.item.links = links2
+        self.assertEqual(links2, self.item.parent_links)
+
     def test_find_ref(self):
         """Verify an item's reference can be found."""
         self.item.ref = "REF" + "123"  # to avoid matching in this file
@@ -313,8 +339,24 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.assertEqual('text.txt', os.path.basename(relpath))
         self.assertEqual(3, line)
 
-    def test_find_rlinks(self):
-        """Verify an item's reverse links can be found."""
+    def test_find_ref_filename(self):
+        """Verify an item's reference can also be a filename."""
+        self.item.ref = "text.txt"
+        relpath, line = self.item.find_ref(FILES)
+        self.assertEqual('text.txt', os.path.basename(relpath))
+        self.assertEqual(None, line)
+
+    def test_find_ref_error(self):
+        """Verify an error occurs when no external reference found."""
+        self.item.ref = "not found".replace(' ', '')  # avoids self match
+        self.assertRaises(DoorstopError, self.item.find_ref, root=EMPTY)
+
+    def test_find_ref_none(self):
+        """Verify nothing returned when no external reference is specified."""
+        self.assertEqual((None, None), self.item.find_ref())
+
+    def test_find_child_objects(self):
+        """Verify an item's child objects can be found."""
 
         mock_document_p = Mock()
         mock_document_p.prefix = 'RQ'
@@ -336,35 +378,25 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
             mock_document_c.__iter__ = mock_iter2
             yield mock_document_c
 
-        self.item.add_link('fake1')
+        self.item.link('fake1')
         mock_tree = Mock()
         mock_tree.__iter__ = mock_iter
         mock_tree.find_item = lambda identifier: Mock(id='fake1')
-        rlinks, childrem = self.item.find_rlinks(mock_document_p, mock_tree)
-        self.assertEqual(['TST001'], rlinks)
-        self.assertEqual([mock_document_c], childrem)
+        self.item.tree = mock_tree
+        self.item.document = mock_document_p
 
-    def test_find_ref_filename(self):
-        """Verify an item's reference can also be a filename."""
-        self.item.ref = "text.txt"
-        relpath, line = self.item.find_ref(FILES)
-        self.assertEqual('text.txt', os.path.basename(relpath))
-        self.assertEqual(None, line)
+        links = self.item.find_child_links()
+        items = self.item.find_child_items()
+        documents = self.item.find_child_documents()
+        self.assertEqual(['TST001'], links)
+        self.assertEqual([mock_item], items)
+        self.assertEqual([mock_document_c], documents)
 
-    def test_find_ref_error(self):
-        """Verify an error occurs when no external reference found."""
-        self.item.ref = "not found".replace(' ', '')  # avoids self match
-        self.assertRaises(DoorstopError, self.item.find_ref, root=EMPTY)
-
-    @unittest.skipUnless(os.getenv(ENV), REASON)
-    def test_find_ref_error_long(self):
-        """Verify an error occurs when no external reference found (long)."""
-        self.item.ref = "not found".replace(' ', '')  # avoids self match
-        self.assertRaises(DoorstopError, self.item.find_ref)
-
-    def test_find_ref_none(self):
-        """Verify nothing returned when no external reference is specified."""
-        self.assertEqual((None, None), self.item.find_ref())
+    def test_find_child_objects_standalone(self):
+        """Verify a standalone item has no child objects."""
+        self.assertEqual([], self.item.child_links)
+        self.assertEqual([], self.item.child_items)
+        self.assertEqual([], self.item.child_documents)
 
     def test_invalid_file_name(self):
         """Verify an invalid file name cannot be a requirement."""
@@ -380,7 +412,9 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
     def test_new(self):
         """Verify items can be created."""
         MockItem._new.reset_mock()
-        item = MockItem.new(EMPTY, FILES, 'TEST', '', 5, 42, (1, 2, 3))
+        item = MockItem.new(None, None,
+                            EMPTY, FILES, 'TEST00042',
+                            level=(1, 2, 3))
         path = os.path.join(EMPTY, 'TEST00042.yml')
         self.assertEqual(path, item.path)
         self.assertEqual((1, 2, 3), item.level)
@@ -390,75 +424,84 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
     def test_new_special(self):
         """Verify items can be created with a specially named prefix."""
         MockItem._new.reset_mock()
-        item = MockItem.new(EMPTY, FILES, 'VSM.HLR_01-002', '-', 3, 42, (1, 0))
+        item = MockItem.new(None, None,
+                            EMPTY, FILES, 'VSM.HLR_01-002-042',
+                            level=(1, 0))
         path = os.path.join(EMPTY, 'VSM.HLR_01-002-042.yml')
         self.assertEqual(path, item.path)
-        self.assertEqual((1, 0), item.level)
+        self.assertEqual((1,), item.level)
         MockItem._new.assert_called_once_with(path, name='item')
 
     def test_new_existing(self):
         """Verify an exception is raised if the item already exists."""
-        self.assertRaises(DoorstopError,
-                          Item.new, FILES, FILES, 'REQ', '', 3, 2, (1, 2, 3))
+        self.assertRaises(DoorstopError, Item.new,
+                          None, None,
+                          FILES, FILES, 'REQ002',
+                          level=(1, 2, 3))
 
-    def test_valid_invalid_ref(self):
+    def test_validate_invalid_ref(self):
         """Verify an invalid reference fails validity."""
         with patch('doorstop.core.item.Item.find_ref',
                    Mock(side_effect=DoorstopError)):
-            self.assertFalse(self.item.valid())
+            self.assertFalse(self.item.validate())
 
-    def test_valid_inactive(self):
+    def test_validate_inactive(self):
         """Verify an inactive item is not checked."""
         self.item.active = False
         with patch('doorstop.core.item.Item.find_ref',
                    Mock(side_effect=DoorstopError)):
-            self.assertTrue(self.item.valid())
+            self.assertTrue(self.item.validate())
 
-    def test_valid_nonnormative_with_links(self):
+    def test_validate_nonnormative_with_links(self):
         """Verify a non-normative item with links can be checked."""
         self.item.normative = False
         self.item.links = ['a']
-        self.assertTrue(self.item.valid())
+        self.assertTrue(self.item.validate())
 
-    def test_valid_link_to_inactive(self):
+    def test_validate_link_to_inactive(self):
         """Verify a link to an inactive item can be checked."""
         mock_item = Mock()
         mock_item.active = False
         mock_tree = MagicMock()
         mock_tree.find_item = Mock(return_value=mock_item)
         self.item.links = ['a']
-        self.assertTrue(self.item.valid(tree=mock_tree))
+        self.item.tree = mock_tree
+        self.assertTrue(self.item.validate())
 
-    def test_valid_link_to_nonnormative(self):
+    def test_validate_link_to_nonnormative(self):
         """Verify a link to an non-normative item can be checked."""
         mock_item = Mock()
         mock_item.normative = False
         mock_tree = MagicMock()
         mock_tree.find_item = Mock(return_value=mock_item)
         self.item.links = ['a']
-        self.assertTrue(self.item.valid(tree=mock_tree))
+        self.item.tree = mock_tree
+        self.assertTrue(self.item.validate())
 
-    def test_valid_document(self):
+    def test_validate_document(self):
         """Verify an item can be checked against a document."""
         mock_document = Mock()
         mock_document.parent = 'fake'
-        self.assertTrue(self.item.valid(document=mock_document))
+        self.item.document = mock_document
+        self.assertTrue(self.item.validate())
 
-    def test_valid_document_with_links(self):
+    def test_validate_document_with_links(self):
         """Verify an item can be checked against a document with links."""
-        self.item.add_link('unknown1')
+        self.item.link('unknown1')
         mock_document = Mock()
         mock_document.parent = 'fake'
-        self.assertTrue(self.item.valid(document=mock_document))
+        self.item.document = mock_document
+        self.assertTrue(self.item.validate())
 
-    def test_valid_document_with_bad_link_IDs(self):
+    def test_validate_document_with_bad_link_IDs(self):
         """Verify an item can be checked against a document w/ bad link IDs."""
-        self.item.add_link('invalid')
+        self.item.link('invalid')
         mock_document = Mock()
         mock_document.parent = 'fake'
-        self.assertFalse(self.item.valid(document=mock_document))
+        self.item.document = mock_document
+        self.assertFalse(self.item.validate())
 
-    def test_valid_tree(self):
+    def test_validate_tree(self):
         """Verify an item can be checked against a tree."""
 
         def mock_iter(self):  # pylint: disable=W0613
@@ -476,22 +519,25 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
             mock_document.__iter__ = mock_iter2
             yield mock_document
 
-        self.item.add_link('fake1')
+        self.item.link('fake1')
 
         mock_tree = Mock()
         mock_tree.__iter__ = mock_iter
         mock_tree.find_item = lambda identifier: Mock(id='fake1')
 
-        self.assertTrue(self.item.valid(tree=mock_tree))
+        self.item.tree = mock_tree
 
-    def test_valid_tree_error(self):
+        self.assertTrue(self.item.validate())
+
+    def test_validate_tree_error(self):
         """Verify an item can be checked against a tree with errors."""
-        self.item.add_link('fake1')
-        tree = MagicMock()
-        tree.find_item = Mock(side_effect=DoorstopError)
-        self.assertFalse(self.item.valid(tree=tree))
+        self.item.link('fake1')
+        mock_tree = MagicMock()
+        mock_tree.find_item = Mock(side_effect=DoorstopError)
+        self.item.tree = mock_tree
+        self.assertFalse(self.item.validate())
 
-    def test_valid_both(self):
+    def test_validate_both(self):
         """Verify an item can be checked against both."""
 
         def mock_iter(seq):
@@ -512,10 +558,12 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         mock_tree = Mock()
         mock_tree.__iter__ = mock_iter([mock_document])
 
-        self.assertTrue(self.item.valid(document=mock_document,
-                                        tree=mock_tree))
+        self.item.document = mock_document
+        self.item.tree = mock_tree
 
-    def test_valid_both_no_reverse_links(self):
+        self.assertTrue(self.item.validate())
+
+    def test_validate_both_no_reverse_links(self):
         """Verify an item can be checked against both (no reverse links)."""
 
         def mock_iter(self):  # pylint: disable=W0613
@@ -533,7 +581,7 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
             mock_document.__iter__ = mock_iter2
             yield mock_document
 
-        self.item.add_link('fake1')
+        self.item.link('fake1')
 
         mock_document = Mock()
         mock_document.prefix = 'RQ'
@@ -542,8 +590,15 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         mock_tree.__iter__ = mock_iter
         mock_tree.find_item = lambda identifier: Mock(id='fake1')
 
-        self.assertTrue(self.item.valid(document=mock_document,
-                                        tree=mock_tree))
+        self.item.document = mock_document
+        self.item.tree = mock_tree
+
+        self.assertTrue(self.item.validate())
+
+    @patch('doorstop.core.item.Item.get_issues', Mock(return_value=[]))
+    def test_issues(self):
+        """Verify an item's issues convenience property can be accessed."""
+        self.assertEqual(0, len(self.item.issues))
 
     @patch('os.remove')
     def test_delete(self, mock_remove):
