@@ -9,10 +9,11 @@ import operator
 import logging
 
 from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
-from doorstop.core.tree import Tree, build
+from doorstop.core.tree import _clear_tree
+from doorstop.core.tree import Tree, build, find_document, find_item
 from doorstop.core.document import Document
 
-from doorstop.core.test import ENV, REASON, FILES, SYS, EMPTY
+from doorstop.core.test import FILES, SYS, EMPTY
 from doorstop.core.test.test_document import MockDocument as _MockDocment
 
 
@@ -46,6 +47,11 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
         b2.children = [c1, c2]
         cls.tree = a
 
+    def test_repr(self):
+        """Verify trees can be represented."""
+        text = "<Tree a <- [ b1, b2 <- [ c1, c2 ] ]>"
+        self.assertEqual(text, repr(self.tree))
+
     def test_str(self):
         """Verify trees can be converted to strings."""
         text = "a <- [ b1, b2 <- [ c1, c2 ] ]"
@@ -69,6 +75,7 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
         child = self.tree.children[1].children[0]
         self.assertIn(child.document, self.tree)
 
+    @patch('doorstop.settings.REORDER', False)
     def test_from_list(self):
         """Verify a tree can be created from a list."""
         a = MockDocument(EMPTY)
@@ -82,7 +89,7 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
         docs = [a, b, c]
         tree = Tree.from_list(docs)
         self.assertEqual(3, len(tree))
-        self.assertTrue(tree.valid())
+        self.assertTrue(tree.validate())
 
     def test_from_list_no_root(self):
         """Verify an error occurs when the tree has no root."""
@@ -118,6 +125,16 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
         docs = [a, b, c]
         self.assertRaises(DoorstopError, Tree.from_list, docs)
 
+    def test_place_no_parent(self):
+        """Verify an error occurs when a node is missing a parent."""
+
+        a = MockDocument(EMPTY)
+        a.prefix = 'A'
+        b = MockDocument(EMPTY)
+        b.prefix = 'B'
+        tree = Tree(a)
+        self.assertRaises(DoorstopError, tree._place, b)  # pylint: disable=W0212
+
 
 @patch('doorstop.core.document.Document', MockDocument)  # pylint: disable=R0904
 @patch('doorstop.core.tree.Document', MockDocument)  # pylint: disable=R0904
@@ -126,14 +143,19 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
     """Unit tests for the Tree class."""  # pylint: disable=C0103
 
     def setUp(self):
-        self.tree = Tree(Document(SYS))
-        self.tree._place(Document(FILES))  # pylint: disable=W0212
+        document = Document(SYS)
+        self.tree = Tree(document)
+        document.tree = self.tree
+        document = Document(FILES)
+        self.tree._place(document)  # pylint: disable=W0212
+        document.tree = self.tree
 
     @patch('doorstop.core.vcs.find_root', Mock(return_value=EMPTY))
     def test_palce_empty(self):
         """Verify a document can be placed in an empty tree."""
         tree = build(EMPTY)
-        doc = MockDocument.new(os.path.join(EMPTY, 'temp'), EMPTY, 'TEMP')
+        doc = MockDocument.new(tree,
+                               os.path.join(EMPTY, 'temp'), EMPTY, 'TEMP')
         tree._place(doc)  # pylint: disable=W0212
         self.assertEqual(1, len(tree))
 
@@ -141,143 +163,171 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
     def test_palce_empty_no_parent(self):
         """Verify a document with parent cannot be placed in an empty tree."""
         tree = build(EMPTY)
-        doc = MockDocument.new(os.path.join(EMPTY, 'temp'), EMPTY, 'TEMP',
+        doc = MockDocument.new(tree,
+                               os.path.join(EMPTY, 'temp'), EMPTY, 'TEMP',
                                parent='REQ')
         self.assertRaises(DoorstopError, tree._place, doc)  # pylint: disable=W0212
 
-    @patch('doorstop.core.document.Document.issues')
-    def test_valid(self, mock_issues):
+    def test_documents(self):
+        """Verify the documents in a tree can be accessed."""
+        documents = self.tree.documents
+        self.assertEqual(2, len(documents))
+        for document in self.tree:
+            logging.debug("document: {}".format(document))
+            self.assertIs(self.tree, document.tree)
+
+    @patch('doorstop.core.document.Document.get_issues')
+    def test_validate(self, mock_get_issues):
         """Verify trees can be checked."""
         logging.info("tree: {}".format(self.tree))
-        self.assertTrue(self.tree.valid())
-        self.assertEqual(2, mock_issues.call_count)
+        self.assertTrue(self.tree.validate())
+        self.assertEqual(2, mock_get_issues.call_count)
 
-    @unittest.skipUnless(os.getenv(ENV), REASON)
-    def test_valid_long(self):
-        """Verify trees can be checked (long)."""
-        logging.info("tree: {}".format(self.tree))
-        self.assertTrue(self.tree.valid())
-
-    def test_valid_no_documents(self):
+    def test_validate_no_documents(self):
         """Verify an empty tree can be checked."""
         tree = Tree(None, root='.')
-        self.assertTrue(tree.valid())
+        self.assertTrue(tree.validate())
 
-    def test_valid_document(self):
-        """Verify an document error fails the tree valid."""
-        mock_issues = Mock(return_value=[DoorstopError('e'),
-                                         DoorstopWarning('w'),
-                                         DoorstopInfo('i')])
-        with patch.object(self.tree, 'issues', mock_issues):
-            self.assertFalse(self.tree.valid())
+    @patch('doorstop.settings.REORDER', False)
+    @patch('doorstop.core.document.Document.get_issues',
+           Mock(return_value=[DoorstopError('error'),
+                              DoorstopWarning('warning'),
+                              DoorstopInfo('info')]))
+    def test_validate_document(self):
+        """Verify a document error fails the tree validation."""
+        self.assertFalse(self.tree.validate())
 
-    @patch('doorstop.core.document.Document.issues', Mock(return_value=[]))
-    def test_valid_hook(self):
+    @patch('doorstop.core.document.Document.get_issues', Mock(return_value=[]))
+    def test_validate_hook(self):
         """Verify a document hook can be called."""
         mock_hook = MagicMock()
-        self.tree.valid(document_hook=mock_hook)
+        self.tree.validate(document_hook=mock_hook)
         self.assertEqual(2, mock_hook.call_count)
 
-    def test_new(self):
-        """Verify a new document can be created on a tree."""
-        self.tree.new(EMPTY, '_TEST', parent='REQ')
+    @patch('doorstop.core.tree.Tree.get_issues', Mock(return_value=[]))
+    def test_issues(self):
+        """Verify an tree's issues convenience property can be accessed."""
+        self.assertEqual(0, len(self.tree.issues))
 
-    def test_new_unknown_parent(self):
+    def test_new_document(self):
+        """Verify a new document can be created on a tree."""
+        self.tree.new_document(EMPTY, '_TEST', parent='REQ')
+
+    def test_new_document_unknown_parent(self):
         """Verify an exception is raised for an unknown parent."""
         temp = tempfile.mkdtemp()
-        self.assertRaises(DoorstopError, self.tree.new,
+        self.assertRaises(DoorstopError, self.tree.new_document,
                           temp, '_TEST', parent='UNKNOWN')
         self.assertFalse(os.path.exists(temp))
 
     @patch('doorstop.core.vcs.git.WorkingCopy.lock')
-    @patch('doorstop.core.document.Document.add')
-    def test_add(self, mock_add, mock_lock):
+    @patch('doorstop.core.document.Document.add_item')
+    def test_add_item(self, mock_add_item, mock_lock):
         """Verify an item can be added to a document."""
-        self.tree.add('REQ')
-        mock_add.assert_called_once_with()
+        self.tree.add_item('REQ')
+        mock_add_item.assert_called_once_with(level=None, reorder=True)
         path = os.path.join(FILES, '.doorstop.yml')
         mock_lock.assert_called_once_with(path)
 
-    def test_add_unknown_prefix(self):
+    @patch('doorstop.core.vcs.git.WorkingCopy.lock')
+    @patch('doorstop.core.document.Document.add_item')
+    def test_add_item_level(self, mock_add, mock_lock):
+        """Verify an item can be added to a document with a level."""
+        self.tree.add_item('REQ', level='1.2.3')
+        mock_add.assert_called_once_with(level='1.2.3', reorder=True)
+        path = os.path.join(FILES, '.doorstop.yml')
+        mock_lock.assert_called_once_with(path)
+
+    def test_add_item_unknown_prefix(self):
         """Verify an exception is raised for an unknown prefix (item)."""
         # Cache miss
-        self.assertRaises(DoorstopError, self.tree.add, 'UNKNOWN')
+        self.assertRaises(DoorstopError, self.tree.add_item, 'UNKNOWN')
         # Cache hit
-        self.assertRaises(DoorstopError, self.tree.add, 'UNKNOWN')
+        self.assertRaises(DoorstopError, self.tree.add_item, 'UNKNOWN')
 
+    @patch('doorstop.settings.REORDER', False)
     @patch('doorstop.core.item.Item.delete')
-    def test_remove(self, mock_delete):
+    def test_remove_item(self, mock_delete):
         """Verify an item can be removed from a document."""
-        self.tree.remove('req1')
+        self.tree.remove_item('req1', reorder=False)
         mock_delete.assert_called_once_with()
 
-    def test_remove_unknown_item(self):
+    def test_remove_item_unknown_item(self):
         """Verify an exception is raised removing an unknown item."""
-        self.assertRaises(DoorstopError, self.tree.remove, 'req9999')
+        self.assertRaises(DoorstopError, self.tree.remove_item, 'req9999')
 
-    @patch('doorstop.core.item.Item.add_link')
-    def test_link(self, mock_add_link):
+    @patch('doorstop.core.item.Item.link')
+    def test_link_items(self, mock_link):
         """Verify two items can be linked."""
-        self.tree.link('req1', 'req2')
-        mock_add_link.assert_called_once_with('REQ002')
+        self.tree.link_items('req1', 'req2')
+        mock_link.assert_called_once_with('REQ002')
 
-    def test_link_unknown_child_prefix(self):
+    def test_link_items_unknown_child_prefix(self):
         """Verify an exception is raised with an unknown child prefix."""
-        self.assertRaises(DoorstopError, self.tree.link, 'unknown1', 'req2')
+        self.assertRaises(DoorstopError,
+                          self.tree.link_items, 'unknown1', 'req2')
 
-    def test_link_unknown_child_number(self):
+    def test_link_items_unknown_child_number(self):
         """Verify an exception is raised with an unknown child number."""
-        self.assertRaises(DoorstopError, self.tree.link, 'req9999', 'req2')
+        self.assertRaises(DoorstopError,
+                          self.tree.link_items, 'req9999', 'req2')
 
-    def test_link_unknown_parent_prefix(self):
+    def test_link_items_unknown_parent_prefix(self):
         """Verify an exception is raised with an unknown parent prefix."""
-        self.assertRaises(DoorstopError, self.tree.link, 'req1', 'unknown1')
+        self.assertRaises(DoorstopError,
+                          self.tree.link_items, 'req1', 'unknown1')
 
-    def test_link_unknown_parent_number(self):
+    def test_link_items_unknown_parent_number(self):
         """Verify an exception is raised with an unknown parent prefix."""
-        self.assertRaises(DoorstopError, self.tree.link, 'req1', 'req9999')
+        self.assertRaises(DoorstopError,
+                          self.tree.link_items, 'req1', 'req9999')
 
-    @patch('doorstop.core.item.Item.remove_link')
-    def test_unlink(self, mock_add_link):
+    @patch('doorstop.core.item.Item.unlink')
+    def test_unlink_items(self, mock_unlink):
         """Verify two items can be unlinked."""
-        self.tree.unlink('req3', 'req1')
-        mock_add_link.assert_called_once_with('REQ001')
+        self.tree.unlink_items('req3', 'req1')
+        mock_unlink.assert_called_once_with('REQ001')
 
-    def test_unlink_unknown_child_prefix(self):
+    def test_unlink_items_unknown_child_prefix(self):
         """Verify an exception is raised with an unknown child prefix."""
-        self.assertRaises(DoorstopError, self.tree.unlink, 'unknown1', 'req1')
+        self.assertRaises(DoorstopError,
+                          self.tree.unlink_items, 'unknown1', 'req1')
 
-    def test_unlink_unknown_child_number(self):
+    def test_unlink_items_unknown_child_number(self):
         """Verify an exception is raised with an unknown child number."""
-        self.assertRaises(DoorstopError, self.tree.unlink, 'req9999', 'req1')
+        self.assertRaises(DoorstopError,
+                          self.tree.unlink_items, 'req9999', 'req1')
 
-    def test_unlink_unknown_parent_prefix(self):
+    def test_unlink_items_unknown_parent_prefix(self):
         """Verify an exception is raised with an unknown parent prefix."""
         # Cache miss
-        self.assertRaises(DoorstopError, self.tree.unlink, 'req3', 'unknown1')
+        self.assertRaises(DoorstopError,
+                          self.tree.unlink_items, 'req3', 'unknown1')
         # Cache hit
-        self.assertRaises(DoorstopError, self.tree.unlink, 'req3', 'unknown1')
+        self.assertRaises(DoorstopError,
+                          self.tree.unlink_items, 'req3', 'unknown1')
 
-    def test_unlink_unknown_parent_number(self):
+    def test_unlink_items_unknown_parent_number(self):
         """Verify an exception is raised with an unknown parent prefix."""
-        self.assertRaises(DoorstopError, self.tree.unlink, 'req3', 'req9999')
+        self.assertRaises(DoorstopError,
+                          self.tree.unlink_items, 'req3', 'req9999')
 
     @patch('doorstop.core.vcs.git.WorkingCopy.lock')
     @patch('doorstop.core.tree._open')
-    def test_edit(self, mock_open, mock_lock):
+    def test_edit_item(self, mock_open, mock_lock):
         """Verify an item can be edited in a tree."""
-        self.tree.edit('req2', launch=True)
+        self.tree.edit_item('req2', launch=True)
         path = os.path.join(FILES, 'REQ002.yml')
         mock_open.assert_called_once_with(path, tool=None)
         mock_lock.assert_called_once_with(path)
 
-    def test_edit_unknown_prefix(self):
+    def test_edit_item_unknown_prefix(self):
         """Verify an exception is raised for an unknown prefix (document)."""
-        self.assertRaises(DoorstopError, self.tree.edit, 'unknown1')
+        self.assertRaises(DoorstopError, self.tree.edit_item, 'unknown1')
 
-    def test_edit_unknown_number(self):
+    def test_edit_item_unknown_number(self):
         """Verify an exception is raised for an unknown number."""
-        self.assertRaises(DoorstopError, self.tree.edit, 'req9999')
+        self.assertRaises(DoorstopError, self.tree.edit_item, 'req9999')
 
     def test_find_item(self):
         """Verify an item can be found by exact ID."""
@@ -334,3 +384,21 @@ class TestModule(unittest.TestCase):  # pylint: disable=R0904
         """Verify documents can be skipped while building a tree."""
         tree = build(FILES)
         self.assertEqual(0, len(tree))
+
+    @patch('doorstop.core.tree.build', Mock(return_value=Tree(Mock())))
+    @patch('doorstop.core.tree.Tree.find_document')
+    def test_find_document(self, mock_find_document):  # pylint: disable=R0201
+        """Verify documents can be found using a convenience function."""
+        _clear_tree()
+        prefix = 'req'
+        find_document(prefix)
+        mock_find_document.assert_called_once_with(prefix)
+
+    @patch('doorstop.core.tree.build', Mock(return_value=Tree(Mock())))
+    @patch('doorstop.core.tree.Tree.find_item')
+    def test_find_item(self, mock_find_item):  # pylint: disable=R0201
+        """Verify items can be found using a convenience function."""
+        _clear_tree()
+        identifier = 'req1'
+        find_item(identifier)
+        mock_find_item.assert_called_once_with(identifier)
