@@ -3,17 +3,41 @@
 import os
 import sys
 import shutil
+import functools
+from itertools import chain
 import subprocess
 import logging
-from itertools import chain
 
-from doorstop.core.base import clear_document_cache, clear_item_cache
-from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
+from doorstop.core.base import BaseValidatable
+from doorstop.core.types import Prefix, ID
+from doorstop.common import DoorstopError, DoorstopWarning
 from doorstop.core.document import Document
 from doorstop.core import vcs
 
 
-class Tree(object):  # pylint: disable=R0902
+def clear_document_cache(func):
+    """Decorator for methods that should clear the document cache."""
+    @functools.wraps(func)
+    def wrapped(self, *args, **kwargs):
+        """Wrapped method to clear document cache after execution."""
+        result = func(self, *args, **kwargs)
+        self._document_cache.clear()  # pylint: disable=W0212
+        return result
+    return wrapped
+
+
+def clear_item_cache(func):
+    """Decorator for methods that should clear the item cache."""
+    @functools.wraps(func)
+    def wrapped(self, *args, **kwargs):
+        """Wrapped method to clear item cache after execution."""
+        result = func(self, *args, **kwargs)
+        self._item_cache.clear()  # pylint: disable=W0212
+        return result
+    return wrapped
+
+
+class Tree(BaseValidatable):  # pylint: disable=R0902
 
     """A bidirectional tree structure to store the hierarchy of documents.
 
@@ -22,9 +46,56 @@ class Tree(object):  # pylint: disable=R0902
 
     """
 
+    @staticmethod
+    def from_list(documents, root=None):
+        """Initialize a new tree from a list of documents.
+
+        @param documents: list of Documents
+        @param root: path to root of the project
+
+        @raise DoorstopError: when the tree cannot be built
+
+        @return: new Tree
+
+        """
+        if not documents:
+            return Tree(document=None, root=root)
+        unplaced = list(documents)
+        for document in list(unplaced):
+            if document.parent is None:
+                logging.info("root of the tree: {}".format(document))
+                tree = Tree(document)
+                document.tree = tree
+                unplaced.remove(document)
+                break
+        else:
+            raise DoorstopError("no root document")
+
+        while unplaced:
+            count = len(unplaced)
+            for document in list(unplaced):
+                if document.parent is None:
+                    logging.info("root of the tree: {}".format(document))
+                    raise DoorstopError("multiple root documents")
+                try:
+                    tree._place(document)  # pylint: disable=W0212
+                except DoorstopError as error:
+                    logging.debug(error)
+                else:
+                    logging.info("added to tree: {}".format(document))
+                    document.tree = tree
+                    unplaced.remove(document)
+
+            if len(unplaced) == count:  # no more documents could be placed
+                logging.debug("unplaced documents: {}".format(unplaced))
+                msg = "unplaced document: {}".format(unplaced[0])
+                raise DoorstopError(msg)
+
+        return tree
+
     def __init__(self, document, parent=None, root=None):
         self.document = document
-        self.root = root or document.root  # allows non-documents in tests
+        self.root = root or document.root  # enables mock testing
         self.parent = parent
         self.children = []
         self._vcs = None
@@ -32,8 +103,11 @@ class Tree(object):  # pylint: disable=R0902
         self._item_cache = {}
         self._document_cache = {}
 
+    def __repr__(self):
+        return "<Tree {}>".format(self)
+
     def __str__(self):
-        # Build parent prefix string (getattr to support testing)
+        # Build parent prefix string (enables mock testing)
         prefix = getattr(self.document, 'prefix', self.document)
         # Build children prefix strings
         children = ", ".join(str(c) for c in self.children)
@@ -57,52 +131,6 @@ class Tree(object):  # pylint: disable=R0902
             yield self.document
         yield from chain(*(iter(c) for c in self.children))
 
-    @staticmethod
-    def from_list(documents, root=None):
-        """Get a new tree from a list of documents.
-
-        @param documents: list of Documents
-        @param root: path to root of the project
-
-        @return: new Tree
-
-        @raise DoorstopError: when the tree cannot be built
-
-        """
-        if not documents:
-            return Tree(document=None, root=root)
-        unplaced = list(documents)
-        for document in list(unplaced):
-            if document.parent is None:
-                logging.debug("added root of tree: {}".format(document))
-                tree = Tree(document)
-                logging.info("root of the tree: {}".format(document))
-                unplaced.remove(document)
-                break
-        else:
-            raise DoorstopError("no root document")
-
-        while unplaced:
-            count = len(unplaced)
-            for document in list(unplaced):
-                if document.parent is None:
-                    logging.info("root of the tree: {}".format(document))
-                    raise DoorstopError("multiple root documents")
-                try:
-                    tree._place(document)  # pylint: disable=W0212
-                except DoorstopError as error:
-                    logging.debug(error)
-                else:
-                    logging.info("added to tree: {}".format(document))
-                    unplaced.remove(document)
-
-            if len(unplaced) == count:  # no more documents could be placed
-                logging.debug("unplaced documents: {}".format(unplaced))
-                msg = "unplaced document: {}".format(unplaced[0])
-                raise DoorstopError(msg)
-
-        return tree
-
     def _place(self, document):
         """Attempt to place the document in the current tree.
 
@@ -112,39 +140,52 @@ class Tree(object):  # pylint: disable=R0902
 
         """
         logging.debug("trying to add '{}'...".format(document))
-        if not self.document:
+        if not self.document:  # tree is empty
 
-            # Tree is empty
             if document.parent:
                 msg = "unknown parent for {}: {}".format(document,
                                                          document.parent)
                 raise DoorstopError(msg)
             self.document = document
 
-        elif (document.parent and
-              document.parent.lower() == self.document.prefix.lower()):
+        elif document.parent:  # tree has documents, document has parent
 
-            # Current document is the parent
-            node = Tree(document, self)
-            self.children.append(node)
+            if document.parent.lower() == self.document.prefix.lower():
 
-        else:
+                # Current document is the parent
+                node = Tree(document, self)
+                self.children.append(node)
 
-            # Search for the parent
-            for child in self.children:
-                try:
-                    child._place(document)  # pylint: disable=W0212
-                except DoorstopError:
-                    pass  # the error is raised later
-                else:
-                    break
             else:
-                msg = "unknown parent for {}: {}".format(document,
-                                                         document.parent)
-                raise DoorstopError(msg)
+
+                # Search for the parent
+                for child in self.children:
+                    try:
+                        child._place(document)  # pylint: disable=W0212
+                    except DoorstopError:
+                        pass  # the error is raised later
+                    else:
+                        break
+                else:
+                    msg = "unknown parent for {}: {}".format(document,
+                                                             document.parent)
+                    raise DoorstopError(msg)
+
+        else:  # tree has documents, but no parent specified for document
+
+            msg = "no parent specified for {}".format(document)
+            logging.info(msg)
+            prefixes = ', '.join(document.prefix for document in self)
+            logging.info("parent options: {}".format(document, prefixes))
+            raise DoorstopError(msg)
 
     # attributes #############################################################
 
+    @property
+    def documents(self):
+        """Get an list of documents in the tree."""
+        return list(self)
+
     @property
     def vcs(self):
         """Get the working copy."""
@@ -156,23 +197,24 @@ class Tree(object):  # pylint: disable=R0902
 
     @clear_document_cache
     @clear_item_cache
-    def new(self, path, prefix, sep=None, digits=None, parent=None):  # pylint: disable=R0913
+    def new_document(self, path, value, sep=None, digits=None, parent=None):  # pylint: disable=R0913
         """Create a new document and add it to the tree.
 
         @param path: directory path for the new document
-        @param prefix: document's prefix
+        @param value: document or prefix
         @param sep: separator between prefix and numbers
         @param digits: number of digits for the document's numbers
         @param parent: parent document's prefix
 
-        @return: newly created and placed Document
-
         @raise DoorstopError: if the document cannot be created
 
+        @return: newly created and placed Document
+
         """
-        document = Document.new(path, self.root, prefix,
-                                sep=sep, digits=digits,
-                                parent=parent)
+        prefix = Prefix(value)
+        document = Document.new(self,
+                                path, self.root, prefix, sep=sep,
+                                digits=digits, parent=parent)
         try:
             self._place(document)
         except DoorstopError:
@@ -181,56 +223,63 @@ class Tree(object):  # pylint: disable=R0902
             if os.path.exists(document.path):
                 shutil.rmtree(document.path)
             raise
+        else:
+            logging.info("added to tree: {}".format(document))
         return document
 
     @clear_item_cache
-    def add(self, prefix):
+    def add_item(self, value, level=None, reorder=True):
         """Add a new item to an existing document by prefix.
 
-        @param prefix: document's prefix
-
-        @return: newly created Item
+        @param value: document or prefix
+        @param level: desired item level
+        @param reorder: update levels of document items
 
         @raise DoorstopError: if the item cannot be created
 
+        @return: newly created Item
+
         """
+        prefix = Prefix(value)
         document = self.find_document(prefix)
         self.vcs.lock(document.config)  # prevents duplicate item IDs
-        item = document.add()
+        item = document.add_item(level=level, reorder=reorder)
         return item
 
     @clear_item_cache
-    def remove(self, identifier):
+    def remove_item(self, value, reorder=True):
         """Remove an item from a document by ID.
 
-        @param identifier: item's ID
-
-        @return: removed Item
+        @param value: item or ID
+        @param reorder: update levels of document items
 
         @raise DoorstopError: if the item cannot be removed
 
+        @return: removed Item
+
         """
+        identifier = ID(value)
         for document in self:
             try:
                 document.find_item(identifier)
             except DoorstopError:
                 pass  # item not found in that document
             else:
-                item = document.remove(identifier)
+                item = document.remove_item(identifier, reorder=reorder)
                 return item
 
         raise DoorstopError("no matching ID: {}".format(identifier))
 
-    def link(self, cid, pid):
+    def link_items(self, cid, pid):
         """Add a new link between two items by IDs.
 
-        @param cid: child item's ID
-        @param pid: parent item's ID
-
-        @return: child Item, parent Item
+        @param cid: child item's ID (or child item)
+        @param pid: parent item's ID (or parent item)
 
         @raise DoorstopError: if the link cannot be created
 
+        @return: child Item, parent Item
+
         """
         logging.info("linking {} to {}...".format(cid, pid))
         # Find child item
@@ -238,19 +287,19 @@ class Tree(object):  # pylint: disable=R0902
         # Find parent item
         parent = self.find_item(pid, _kind='parent')
         # Add link
-        child.add_link(parent.id)
+        child.link(parent.id)
         return child, parent
 
-    def unlink(self, cid, pid):
+    def unlink_items(self, cid, pid):
         """Remove a link between two items by IDs.
 
-        @param cid: child item's ID
-        @param pid: parent item's ID
-
-        @return: child Item, parent Item
+        @param cid: child item's ID (or child item)
+        @param pid: parent item's ID (or parent item)
 
         @raise DoorstopError: if the link cannot be removed
 
+        @return: child Item, parent Item
+
         """
         logging.info("unlinking '{}' from '{}'...".format(cid, pid))
         # Find child item
@@ -258,18 +307,20 @@ class Tree(object):  # pylint: disable=R0902
         # Find parent item
         parent = self.find_item(pid, _kind='parent')
         # Remove link
-        child.remove_link(parent.id)
+        child.unlink(parent.id)
         return child, parent
 
-    def edit(self, identifier, tool=None, launch=False):
+    def edit_item(self, identifier, tool=None, launch=False):
         """Open an item for editing by ID.
 
-        @param identifier: ID of item to edit
+        @param identifier: item's ID (or item)
         @param tool: alternative text editor to open the item
-        @param launch: open the default text editor
+        @param launch: open the text editor
 
         @raise DoorstopError: if the item cannot be found
 
+        @return: edited Item
+
         """
         logging.debug("looking for {}...".format(identifier))
         # Find item
@@ -284,16 +335,17 @@ class Tree(object):  # pylint: disable=R0902
         # Return the item
         return item
 
-    def find_document(self, prefix):
+    def find_document(self, value):
         """Get a document by its prefix.
 
-        @param prefix: document's prefix
-
-        @return: matching Document
+        @param value: document or prefix
 
         @raise DoorstopError: if the document cannot be found
 
+        @return: matching Document
+
         """
+        prefix = Prefix(value)
         logging.debug("looking for document '{}'...".format(prefix))
         try:
             document = self._document_cache[prefix]
@@ -313,16 +365,17 @@ class Tree(object):  # pylint: disable=R0902
 
         raise DoorstopError("no matching prefix: {}".format(prefix))
 
-    def find_item(self, identifier, _kind=''):
+    def find_item(self, value, _kind=''):
         """Get an item by its ID.
 
-        @param identifier: item ID
-
-        @return: matching Item
+        @param value: item or ID
 
         @raise DoorstopError: if the item cannot be found
 
+        @return: matching Item
+
         """
+        identifier = ID(value)
         _kind = (' ' + _kind) if _kind else _kind  # for logging messages
         logging.debug("looking for{} item '{}'...".format(_kind, identifier))
         try:
@@ -347,32 +400,7 @@ class Tree(object):  # pylint: disable=R0902
 
         raise DoorstopError("no matching{} ID: {}".format(_kind, identifier))
 
-    def valid(self, document_hook=None, item_hook=None):
-        """Check the tree (and its documents) for validity.
-
-        @param document_hook: function to call for custom document validation
-        @param item_hook: function to call for custom item validation
-
-        @return: indication that the tree is valid
-
-        """
-        valid = True
-        logging.info("checking tree...")
-        # Display all issues
-        for issue in self.issues(document_hook=document_hook,
-                                 item_hook=item_hook):
-            if isinstance(issue, DoorstopInfo):
-                logging.info(issue)
-            elif isinstance(issue, DoorstopWarning):
-                logging.warning(issue)
-            else:
-                assert isinstance(issue, DoorstopError)
-                logging.error(issue)
-                valid = False
-        # Return the result
-        return valid
-
-    def issues(self, document_hook=None, item_hook=None):
+    def get_issues(self, document_hook=None, item_hook=None):
         """Yield all the tree's issues.
 
         @param document_hook: function to call for custom document validation
@@ -381,16 +409,15 @@ class Tree(object):  # pylint: disable=R0902
         @return: generator of DoorstopError, DoorstopWarning, DoorstopInfo
 
         """
+        hook = document_hook if document_hook else lambda **kwargs: []
         documents = list(self)
         # Check for documents
         if not documents:
             yield DoorstopWarning("no documents")
         # Check each document
         for document in documents:
-            for issue in chain(document_hook(document=document, tree=self)
-                               if document_hook else [],
-                               document.issues(tree=self,
-                                               item_hook=item_hook)):
+            for issue in chain(hook(document=document, tree=self),
+                               document.get_issues(item_hook=item_hook)):
                 # Prepend the document's prefix to yielded exceptions
                 if isinstance(issue, Exception):
                     yield type(issue)("{}: {}".format(document.prefix, issue))
@@ -421,7 +448,7 @@ class Tree(object):  # pylint: disable=R0902
         self.children = []
 
 
-def _open(path, tool=None):  # pragma: no cover, integration test
+def _open(path, tool=None):  # pragma: no cover (integration test)
     """Open the text file using the default editor."""
     if tool:
         args = [tool, path]
@@ -446,10 +473,10 @@ def build(cwd=None, root=None):
     @param cwd: current working directory
     @param root: path to root of the working copy
 
-    @return: new Tree
-
     @raise DoorstopError: when the tree cannot be built
 
+    @return: new Tree
+
     """
     documents = []
 
@@ -483,7 +510,7 @@ def _document_from_path(path, root, documents):
 
     """
     try:
-        document = Document(path, root)
+        document = Document(path, root, tree=None)  # tree attached later
     except DoorstopError:
         pass  # no document in directory
     else:
@@ -493,24 +520,41 @@ def _document_from_path(path, root, documents):
             logging.info("found document: {}".format(document))
             documents.append(document)
 
+
 # convenience functions ######################################################
 
-_TREE = None  # implicitly created tree created for convenience functions
+_tree = None  # implicit tree for convenience functions, pylint:disable=C0103
+
+
+def _get_tree():
+    """Get a shared tree for convenience functions."""
+    global _tree  # pylint: disable=W0603,C0103
+    if _tree is None:
+        _tree = build()
+    return _tree
+
+
+def _set_tree(value):
+    """Set the shared tree to a specific value (for testing)."""
+    global _tree  # pylint: disable=W0603,C0103
+    _tree = value
+
+
+def _clear_tree():
+    """Force the shared tree to be rebuilt."""
+    global _tree  # pylint: disable=W0603,C0103
+    _tree = None
 
 
 def find_document(prefix):
     """Find a document without an explicitly building a tree."""
-    global _TREE  # pylint: disable=W0603
-    if _TREE is None:
-        _TREE = build()
-    document = _TREE.find_document(prefix)
+    tree = _get_tree()
+    document = tree.find_document(prefix)
     return document
 
 
 def find_item(identifier):
     """Find an item without an explicitly building a tree."""
-    global _TREE  # pylint: disable=W0603
-    if _TREE is None:
-        _TREE = build()
-    item = _TREE.find_item(identifier)
+    tree = _get_tree()
+    item = tree.find_item(identifier)
     return item
