@@ -1,16 +1,14 @@
 #!/usr/bin/env python
 
-"""
-Graphical interface for Doorstop.
-"""
+"""Graphical interface for Doorstop."""
 
 import sys
 from unittest.mock import Mock
-try:  # pragma: no cover - not measurable
+try:  # pragma: no cover (manual test)
     import tkinter as tk
     from tkinter import ttk
     from tkinter import font, filedialog
-except ImportError as _exc:  # pragma: no cover, not measurable
+except ImportError as _exc:  # pragma: no cover (manual test)
     sys.stderr.write("WARNING: {}\n".format(_exc))
     tk = Mock()  # pylint: disable=C0103
     ttk = Mock()  # pylint: disable=C0103
@@ -20,19 +18,22 @@ import functools
 from itertools import chain
 import logging
 
-
-from doorstop import GUI, __project__, __version__
-from doorstop.common import SHARED, WarningFormatter, DoorstopError
+from doorstop.common import HelpFormatter, WarningFormatter, DoorstopError
 from doorstop.core import vcs
 from doorstop.core import tree
 from doorstop import settings
 
 
 def main(args=None):
-    """Process command-line arguments and run the program.
-    """
+    """Process command-line arguments and run the program."""
+    from doorstop import GUI, VERSION
     # Main parser
-    parser = argparse.ArgumentParser(prog=GUI, description=__doc__, **SHARED)
+    debug = argparse.ArgumentParser(add_help=False)
+    debug.add_argument('-V', '--version', action='version', version=VERSION)
+    debug.add_argument('-v', '--verbose', action='count', default=0,
+                       help="enable verbose logging")
+    shared = {'formatter_class': HelpFormatter, 'parents': [debug]}
+    parser = argparse.ArgumentParser(prog=GUI, description=__doc__, **shared)
     # Hidden argument to override the root sharing directory path
     parser.add_argument('-j', '--project', metavar="PATH",
                         help="path to the root of the project")
@@ -58,7 +59,6 @@ def main(args=None):
 
 def _configure_logging(verbosity=0):
     """Configure logging using the provided verbosity level (0+)."""
-
     # Configure the logging level and format
     if verbosity == 0:
         level = settings.VERBOSE_LOGGING_LEVEL
@@ -81,13 +81,14 @@ def _run(args, cwd, error):
     @param args: Namespace of CLI arguments (from this module or the CLI)
     @param cwd: current working directory
     @param error: function to call for CLI errors
-    """
 
+    """
+    from doorstop import __project__, __version__
     # Exit if tkinter is not available
     if isinstance(tk, Mock) or isinstance(ttk, Mock):
         return error("tkinter is not available")
 
-    else:  # pragma: no cover, manual test
+    else:  # pragma: no cover (manual test)
 
         root = tk.Tk()
         root.title("{} ({})".format(__project__, __version__))
@@ -101,8 +102,8 @@ def _run(args, cwd, error):
         return True
 
 
-# TODO: remove this function when no longer used
-def _log(func):  # pragma: no cover, manual test
+# TODO: cleanup: remove this logging decorator when no longer used
+def _log(func):  # pragma: no cover (manual test)
     """Decorator for methods that should log calls."""
     @functools.wraps(func)
     def wrapped(self, *args, **kwargs):
@@ -117,7 +118,8 @@ def _log(func):  # pragma: no cover, manual test
     return wrapped
 
 
-class Listbox2(tk.Listbox):  # pragma: no cover, manual test, pylint: disable=R0901,R0904
+class Listbox2(tk.Listbox):  # pragma: no cover (manual test), pylint: disable=R0901,R0904
+
     """Listbox class with automatic width adjustment."""
 
     def autowidth(self, maxwidth=250):
@@ -135,7 +137,8 @@ class Listbox2(tk.Listbox):  # pragma: no cover, manual test, pylint: disable=R0
             self.config(width=width + shift)
 
 
-class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=R0901,R0902,R0904
+class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable=R0901,R0902,R0904
+
     """Graphical application for Doorstop."""
 
     def __init__(self, root, cwd, project):
@@ -192,9 +195,8 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
         # Start the application
         root.after(500, self.find)
 
-    def init(self, root):  # pylint: disable=R0914
+    def init(self, root):  # pylint: disable=R0912,R0914
         """Initialize and return the main frame."""  # pylint: disable=C0301
-
         # Shared arguments
         width_outline = 20
         width_text = 40
@@ -230,7 +232,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
         # Create widgets
         def frame_project(root):
             """Frame for the current project."""
-
             # Configure grid
             frame = ttk.Frame(root, **kw_f)
             frame.rowconfigure(0, weight=1)
@@ -247,7 +248,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
         def frame_tree(root):
             """Frame for the current document."""
-
             # Configure grid
             frame = ttk.Frame(root, **kw_f)
             frame.rowconfigure(0, weight=1)
@@ -265,7 +265,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
         def frame_document(root):
             """Frame for current document's outline and items."""
-
             # Configure grid
             frame = ttk.Frame(root, **kw_f)
             frame.rowconfigure(0, weight=0)
@@ -308,7 +307,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
         def frame_item(root):
             """Frame for the currently selected item."""
-
             # Configure grid
             frame = ttk.Frame(root, **kw_f)
             frame.rowconfigure(0, weight=0)
@@ -368,7 +366,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
         def frame_family(root):
             """Frame for the parent and child document items."""
-
             # Configure grid
             frame = ttk.Frame(root, **kw_f)
             frame.rowconfigure(0, weight=0)
@@ -398,7 +395,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
     def find(self):
         """Find the root of the project."""
-
         if not self.stringvar_project.get():
             try:
                 path = vcs.find_root(self.cwd)
@@ -410,7 +406,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
     @_log
     def browse(self):
         """Browse for the root of a project."""
-
         path = filedialog.askdirectory()
         logging.debug("path: {}".format(path))
         if path:
@@ -418,13 +413,13 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
     def display_tree(self, *_):
         """Display the currently selected tree."""
-
         # Set the current tree
         self.tree = tree.build(root=self.stringvar_project.get())
         logging.info("displaying tree...")
 
         # Display the documents in the tree
-        values = [document.prefix_relpath for document in self.tree]
+        values = ["{} ({})".format(document.prefix, document.relpath)
+                  for document in self.tree]
         self.combobox_documents['values'] = values
 
         # Select the first document
@@ -432,7 +427,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
     def display_document(self, *_):
         """Display the currently selected document."""
-
         # Set the current document
         index = self.combobox_documents.current()
         self.document = list(self.tree)[index]
@@ -462,7 +456,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
     def display_item(self, *_):
         """Display the currently selected item."""
-
         self.ignore = True
 
         # Set the current item
@@ -510,7 +503,7 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
         # Display the items this item has links from
         self.text_children.delete('1.0', 'end')
-        identifiers = self.item.find_rlinks(self.document, self.tree)[0]
+        identifiers = self.item.find_child_links()
         for identifier in identifiers:
             item = self.tree.find_item(identifier)
             text = item.text or item.ref or '???'
@@ -523,7 +516,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
     @_log
     def display_extended(self, *_):
         """Display the currently selected extended attribute."""
-
         self.ignore = True
 
         name = self.stringvar_extendedkey.get()
@@ -585,7 +577,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
     def link(self):
         """Add the specified link to the current item."""
-
         # Add the specified link to the list
         identifier = self.stringvar_link.get()
         if identifier:
@@ -597,7 +588,6 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
 
     def unlink(self):
         """Remove the currently selected link from the current item."""
-
         # Remove the selected link from the list
         index = self.listbox_links.curselection()
         self.listbox_links.delete(index)
@@ -606,5 +596,5 @@ class Application(ttk.Frame):  # pragma: no cover, manual test, pylint: disable=
         self.update_item()
 
 
-if __name__ == '__main__':  # pragma: no cover - manual test
+if __name__ == '__main__':  # pragma: no cover (manual test)
     main()
