@@ -18,5 +18,10 @@ MAX_LINE_LENTH = 79  # line length to trigger multiline on extended attributes
 
 # Validation settings
 REFORMAT = True  # reformat item files during validation
+REORDER = False  # reorder document levels during validation
+CHECK_LEVELS = True  # validate document levels during validation
 CHECK_REF = True  # validate external file references
-CHECK_RLINKS = True  # validate reverse links
+CHECK_CHILD_LINKS = True  # validate reverse links
+
+# Publishing settings
+PUBLISH_CHILD_LINKS = False
